package com.minerarcana.floralchemy.block;

import java.util.Optional;
import java.util.Random;

import com.minerarcana.floralchemy.tileentity.TileEntityFloodedSoil;
import com.teamacronymcoders.base.blocks.BlockTEBase;

import net.minecraft.block.BlockBush;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockRenderLayer;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fluids.FluidRegistry;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.FluidUtil;
import net.minecraftforge.fluids.IFluidTank;
import net.minecraftforge.fluids.capability.CapabilityFluidHandler;
import net.minecraftforge.fluids.capability.IFluidHandler;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public class BlockFloodedSoil extends BlockTEBase<TileEntityFloodedSoil> {

	public static final int FLUID_TRANSFER_AMOUNT_MB = 100;
	public static final int CULTIVATION_FLUID_USE_MB = 100;
	
    public BlockFloodedSoil() {
        super(Material.field_151578_c, "flooded_soil");
        func_149675_a(true);
    }
    
    @Override
    public boolean canSustainPlant(IBlockState state, IBlockAccess world, BlockPos pos, EnumFacing direction, net.minecraftforge.common.IPlantable plantable)
    {
    	IBlockState plant = plantable.getPlant(world, pos.func_177972_a(direction));
    	if(plant.func_177230_c() instanceof BlockBaseBush) {
			TileEntity tile = world.func_175625_s(pos);
        	if(tile instanceof TileEntityFloodedSoil) {
        		TileEntityFloodedSoil soil = (TileEntityFloodedSoil)tile;
        		IFluidTank tank = (IFluidTank)soil.getCapability(CapabilityFluidHandler.FLUID_HANDLER_CAPABILITY, EnumFacing.UP);
        		if(tank != null && tank.getFluidAmount() > 0) {
        			if(((BlockBaseBush)plant.func_177230_c()).cultivatingFluidNames.contains(FluidRegistry.getFluidName(tank.getFluid().getFluid()))) {
        				return true;
        			}
        		}
        	}
    	}
    	return super.canSustainPlant(state, world, pos, direction, plantable);
    }

    @Override
    public void func_180650_b(World worldIn, BlockPos pos, IBlockState state, Random rand) {
        if(!worldIn.field_72995_K) {
            getTileEntity(worldIn, pos).ifPresent(tile -> {
                if(tile.tank.getFluidAmount() >= FLUID_TRANSFER_AMOUNT_MB * 2) {
                    for(EnumFacing facing : EnumFacing.field_82609_l) {
                        if(!EnumFacing.UP.equals(facing)) {
                            BlockPos adjacent = pos.func_177972_a(facing);
                            IBlockState old = worldIn.func_180495_p(adjacent);
                            if(old.func_177230_c() == this) {
                                IFluidHandler otherTank = worldIn.func_175625_s(adjacent)
                                        .getCapability(CapabilityFluidHandler.FLUID_HANDLER_CAPABILITY, facing);
                                FluidStack test = FluidUtil.tryFluidTransfer(otherTank, tile.tank, FLUID_TRANSFER_AMOUNT_MB, false);
                                if(test != null && test.amount == FLUID_TRANSFER_AMOUNT_MB) {
                                    FluidUtil.tryFluidTransfer(otherTank, tile.tank, FLUID_TRANSFER_AMOUNT_MB, true);
                                    worldIn.markAndNotifyBlock(adjacent, worldIn.func_175726_f(adjacent), old,
                                            worldIn.func_180495_p(adjacent), 3);
                                    worldIn.func_175625_s(adjacent).func_70296_d();
                                    tile.func_70296_d();
                                    break;
                                }
                            }
                        }
                    }
                }
            });
        }
    }

    @Override
    public boolean func_149662_c(IBlockState state) {
        return false;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public BlockRenderLayer func_180664_k() {
        return BlockRenderLayer.CUTOUT;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public boolean func_176225_a(IBlockState blockState, IBlockAccess blockAccess, BlockPos pos,
            EnumFacing side) {
        return blockAccess.func_180495_p(pos.func_177972_a(side)).func_177230_c() == this ? false
                : super.func_176225_a(blockState, blockAccess, pos, side);
    }

    @Override
    public boolean func_180639_a(World worldIn, BlockPos pos, IBlockState state, EntityPlayer playerIn,
            EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ) {
        if(playerIn.func_184586_b(hand).hasCapability(CapabilityFluidHandler.FLUID_HANDLER_ITEM_CAPABILITY, null)) {
            Optional<TileEntityFloodedSoil> te = getTileEntity(worldIn, pos);
            if(te.isPresent()) {
                if(FluidUtil.interactWithFluidHandler(playerIn, hand, worldIn, pos, facing)) {
                    te.get().sendBlockUpdate();
                    return true;
                }
            }
        }
        return super.func_180639_a(worldIn, pos, state, playerIn, hand, facing, hitX, hitY, hitZ);
    }

    @Override
    public Class<TileEntityFloodedSoil> getTileEntityClass() {
        return TileEntityFloodedSoil.class;
    }

    @Override
    public TileEntity createTileEntity(World world, IBlockState blockState) {
        return new TileEntityFloodedSoil();
    }
}
