package com.minerarcana.floralchemy.block;

import java.util.Random;

import com.minerarcana.floralchemy.Floralchemy;
import com.minerarcana.floralchemy.item.ItemBlockTinted;
import com.teamacronymcoders.base.blocks.BlockBase;
import com.teamacronymcoders.base.blocks.IHasBlockColor;
import com.teamacronymcoders.base.util.ItemStackUtils;

import net.minecraft.block.Block;
import net.minecraft.block.BlockFenceGate;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.*;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.EnumDyeColor;
import net.minecraft.item.ItemStack;
import net.minecraft.util.*;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.*;
import net.minecraft.world.biome.BiomeColorHelper;
import net.minecraft.world.storage.loot.LootContext;
import net.minecraft.world.storage.loot.LootTable;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public class BlockHedge extends BlockBase implements IHasBlockColor {

    private final boolean isThorny;

    public static final PropertyBool NORTH = PropertyBool.func_177716_a("north");
    public static final PropertyBool EAST = PropertyBool.func_177716_a("east");
    public static final PropertyBool SOUTH = PropertyBool.func_177716_a("south");
    public static final PropertyBool WEST = PropertyBool.func_177716_a("west");

    public BlockHedge(String name, boolean isThorns) {
        super(Material.field_151584_j, name);
        func_180632_j(field_176227_L.func_177621_b().func_177226_a(NORTH, Boolean.valueOf(false))
                .func_177226_a(EAST, Boolean.valueOf(false)).func_177226_a(SOUTH, Boolean.valueOf(false))
                .func_177226_a(WEST, Boolean.valueOf(false)));
        func_149675_a(true);
        func_149647_a(CreativeTabs.field_78031_c);
        func_149711_c(0.2F);
        isThorny = isThorns;
        setItemBlock(new ItemBlockTinted<BlockHedge>(this));
    }

    @Override
    public void getDrops(net.minecraft.util.NonNullList<ItemStack> drops, net.minecraft.world.IBlockAccess world,
            BlockPos pos, IBlockState state, int fortune) {
        if(world instanceof WorldServer) {
            WorldServer serverWorld = (WorldServer) world;
            String location = isThorny ? "block/thorny_hedge" : "block/hedge";
            LootTable table = serverWorld.func_184146_ak()
                    .func_186521_a(new ResourceLocation(Floralchemy.MOD_ID, location));
            LootContext ctx = new LootContext.Builder(serverWorld).func_186469_a(fortune).func_186471_a();
            drops.addAll(table.func_186462_a(serverWorld.field_73012_v, ctx));
        }
        else {
            super.getDrops(drops, world, pos, state, fortune);
        }
    }

    @Override
    public void func_180634_a(World worldIn, BlockPos pos, IBlockState state, Entity entityIn) {
        if(isThorny) {
            entityIn.func_70097_a(DamageSource.field_76367_g, 3.0F);
        }
    }

    @Override
    protected BlockStateContainer func_180661_e() {
        return new BlockStateContainer(this, new IProperty[] { NORTH, EAST, WEST, SOUTH });
    }

    @Override
    public void func_180650_b(World worldIn, BlockPos pos, IBlockState state, Random rand) {
        if(!worldIn.field_72995_K) {
            boolean hasNeighbour = false;
            for(EnumFacing facing : EnumFacing.field_82609_l) {
                BlockPos adjacent = pos.func_177972_a(facing);
                if(worldIn.func_180495_p(adjacent).func_177230_c() == this
                        || worldIn.func_180495_p(adjacent).func_185904_a() == Material.field_151577_b
                        || worldIn.func_180495_p(adjacent).func_185904_a() == Material.field_151578_c) {
                    hasNeighbour = true;
                }
            }
            if(!hasNeighbour) {
                worldIn.func_175698_g(pos);
            }
        }
    }

    @Override
    public AxisAlignedBB func_180646_a(IBlockState blockState, IBlockAccess worldIn, BlockPos pos) {
        return field_185505_j.func_186664_h(0.05F);
    }

    @Override
    public int func_176201_c(IBlockState state) {
        return 0;
    }

    @Override
    public IBlockState func_176221_a(IBlockState state, IBlockAccess worldIn, BlockPos pos) {
        return state.func_177226_a(NORTH, canThisConnectTo(worldIn, pos, EnumFacing.NORTH))
                .func_177226_a(EAST, canThisConnectTo(worldIn, pos, EnumFacing.EAST))
                .func_177226_a(SOUTH, canThisConnectTo(worldIn, pos, EnumFacing.SOUTH))
                .func_177226_a(WEST, canThisConnectTo(worldIn, pos, EnumFacing.WEST));
    }

    private boolean canThisConnectTo(IBlockAccess world, BlockPos pos, EnumFacing facing) {
        BlockPos other = pos.func_177972_a(facing);
        Block block = world.func_180495_p(other).func_177230_c();
        return block.canBeConnectedTo(world, other, facing.func_176734_d())
                || canConnectTo(world, other, facing.func_176734_d());
    }

    public boolean canConnectTo(IBlockAccess worldIn, BlockPos pos, EnumFacing facing) {
        IBlockState iblockstate = worldIn.func_180495_p(pos);
        BlockFaceShape blockfaceshape = iblockstate.func_193401_d(worldIn, pos, facing);
        Block block = iblockstate.func_177230_c();
        boolean flag = blockfaceshape == BlockFaceShape.MIDDLE_POLE
                && (iblockstate.func_185904_a() == field_149764_J || block instanceof BlockFenceGate);
        return !isExcepBlockForAttachWithPiston(block) && blockfaceshape == BlockFaceShape.SOLID || flag;
    }

    public static boolean isExcepBlockForAttachWithPiston(Block block) {
        return Block.func_193382_c(block) || block == Blocks.field_180401_cv || block == Blocks.field_150440_ba
                || block == Blocks.field_150423_aK || block == Blocks.field_150428_aP;
    }

    @Override
    public boolean func_149662_c(IBlockState state) {
        return false;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public BlockRenderLayer func_180664_k() {
        return BlockRenderLayer.CUTOUT;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public boolean func_176225_a(IBlockState blockState, IBlockAccess blockAccess, BlockPos pos,
            EnumFacing side) {
        return blockAccess.func_180495_p(pos.func_177972_a(side)).func_177230_c() == this ? false
                : super.func_176225_a(blockState, blockAccess, pos, side);
    }

    @Override
    public int colorMultiplier(IBlockState state, IBlockAccess world, BlockPos pos, int tintIndex) {
        return world != null && pos != null ? BiomeColorHelper.func_180287_b(world, pos)
                : ColorizerFoliage.func_77468_c();
    }

    @Override
    public boolean func_180639_a(World worldIn, BlockPos pos, IBlockState state, EntityPlayer playerIn,
            EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ) {
        if(!worldIn.field_72995_K) {
            if(ItemStackUtils.doItemsMatch(playerIn.func_184586_b(hand), Items.field_151100_aR)
                    && playerIn.func_184586_b(hand).func_77952_i() == EnumDyeColor.WHITE.func_176767_b()) {
                BlockPos toGrow = pos.func_177972_a(facing);
                if(worldIn.func_175623_d(toGrow)) {
                    worldIn.func_175656_a(toGrow, state);
                    if(!playerIn.field_71075_bZ.field_75098_d) {
                        playerIn.func_184586_b(hand).func_190918_g(1);
                    }
                    worldIn.func_175718_b(2005, toGrow, 0);
                    return true;
                }
            }
        }
        return false;
    }
}
