package com.minerarcana.floralchemy.worldgen;

import java.util.Random;

import com.minerarcana.floralchemy.FloraObjectHolder;
import com.minerarcana.floralchemy.block.BlockBaseBush;
import com.minerarcana.floralchemy.block.BlockCindermoss;

import net.minecraft.block.BlockDirt;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraft.world.chunk.IChunkProvider;
import net.minecraft.world.gen.IChunkGenerator;
import net.minecraftforge.common.BiomeDictionary;
import net.minecraftforge.fml.common.IWorldGenerator;

public class FloralchemyWorldGenerator implements IWorldGenerator {

    @Override
    public void generate(Random random, int chunkX, int chunkZ, World world, IChunkGenerator chunkGenerator,
            IChunkProvider chunkProvider) {
        switch(world.field_73011_w.getDimension()) {
            case -1:
                generateNether(world, random, chunkX * 16, chunkZ * 16);
                break;
            case 0:
                generateSurface(world, random, chunkX * 16, chunkZ * 16);
                break;
            case 1:
                break;
        }

    }

    private void generateSurface(World world, Random random, int x, int z) {
        if(random.nextInt(15) == 0) {
            BlockPos position = new BlockPos(x + 8, random.nextInt(world.func_72940_L()), z + 8);
            if(BiomeDictionary.hasType(world.func_180494_b(position), BiomeDictionary.Type.FOREST)) {
                while(world.func_175623_d(position) && position.func_177956_o() > 0) {
                    position = position.func_177977_b();
                }

                for(int i = 0; i < 3; ++i) {
                    BlockPos blockpos = position.func_177982_a(random.nextInt(8) - random.nextInt(8),
                            random.nextInt(4) - random.nextInt(4), random.nextInt(8) - random.nextInt(8));
                    if(world.func_175623_d(blockpos) && BlockDirt.DirtType.PODZOL
                            .equals(world.func_180495_p(blockpos.func_177977_b()).func_177228_b().get(BlockDirt.field_176386_a))) {
                        world.func_180501_a(blockpos, FloraObjectHolder.DEVILSNARE.func_176223_P(), 2);
                    }
                }
            }
        }
    }

    private void generateNether(World world, Random random, int x, int z) {
        if(random.nextInt(30) == 0) {
            this.generateBush(world, random, new BlockPos(x + 8, random.nextInt(40), z + 8),
                    (BlockCindermoss) FloraObjectHolder.CINDERMOSS);
        }
    }

    public <B extends BlockBaseBush> boolean generateBush(World world, Random random, BlockPos position, B bush) {
        while(world.func_175623_d(position) && position.func_177956_o() > 0) {
            position = position.func_177977_b();
        }

        for(int i = 0; i < 7; ++i) {
            BlockPos blockpos = position.func_177982_a(random.nextInt(8) - random.nextInt(8),
                    random.nextInt(4) - random.nextInt(4), random.nextInt(8) - random.nextInt(8));
            if(world.func_175623_d(blockpos) && ((B) bush).canSustainPlant(world.func_180495_p(blockpos.func_177977_b()), world, blockpos, EnumFacing.UP, bush)) {
                world.func_180501_a(blockpos, bush.func_176223_P(), 2);
            }
        }

        return true;
    }
}
